/*
By y.monmagnon@wanadoo.fr.
Work in progress, subject to modifications without notice.
Check periodically !
Have fun.

Yves.
*/

// Need G4P library
import g4p_controls.*;

Plot p;  //Globals
Tdf tdf;
//GPanel pnl;
GSketchPad spad;
PGraphics pg;
GLabel LbPf; 
String cFN = "TCT3.conf";
StringDict config;

public void setup(){
  tdf = new Tdf();
  p = new Plot();
  config = new StringDict();
  loadConfig(cFN);  //Path to the config file should be "argv" or so !
  size(int(config.get("winw")),int(config.get("winh")),JAVA2D);
  frameRate(10);
  createGUI();
  customGUI();
  if (config.hasKey("TDFpath")) tdf.loadFile();
}

public void customGUI(){
// Create the graphic for plotting
  pg = createGraphics(int(config.get("plotw")),int(config.get("ploth")), JAVA2D);
  // Create the G4P control to position and display the graphic
  spad = new GSketchPad(this,int(config.get("plotl")),int(config.get("plott")),int(config.get("plotw")),int(config.get("ploth")));
  spad.setGraphic(pg);
// Resize and Move sliders  
// The horizontal one:
//  custom_slider1.moveTo(100,100,GControlMode.CENTER);
// The vertical one:  
  custom_slider2.moveTo(float(config.get("plotl"))-5,
                        0.5*(float(config.get("plott")) + float(config.get("ploth")))+5,GControlMode.CENTER);
  p.init();
  p.graticule();
}

// G4P code for folder and file dialogs
public void handleFileDialog() {
  String fname;
  boolean b;
  
    // Use file filter if possible
  fname = G4P.selectInput("Select TDF");
//  println(fname);
  if (fname != null) {
    tdf.setPath(fname);
    tdf.loadFile();
  }
}

// Returns a "Short Human Readable" representation of a value.
/*  Exemples:
                        3 significant digits only !
0.000 012 345  12.3µ    12µ3
0.000 123 4    123µ     
0.001 23       1.23m    1m23
0.0123         12.3m    12m3 
0.012          12m
0.1            100m
0.15           150m
0.95           950m
0.951          951m     0.95
1              1
12             12
12.3           12.3
12.34          12.3
12.9           12.9
123            123
1 234          1.23K    1K23
12 345         12.3K    12K3
123  456       123K
1 234 567      1.23M    1M23
12 345 678     12.3M     12M3 
123 456 789    123M    
1 234 567 890  1.23G    1G23
*/
String fixDsep(float v,int digits,String dsep){
  String s;
  String[] intdec;

  s = str(v);
//  println ("fixDsep",s,dsep);
//  if ((dsep == " ")|(dsep == "")) dsep = ".";  // insure there is a separator
  intdec = split(s,".");  //Separate integer and decimal parts
//  println(intdec[0]+intdec[1]);
  if (float(intdec[0]) == 0) intdec[0] = "";  //integer part is 0, remove it
  if (float(intdec[1]) == 0) intdec[1] = "";  //decimal part is 0, remove it
  if (intdec[0].length() > digits) return "??";  //too large
  while ((intdec[0].length())+(intdec[1].length()) > digits){
   intdec[1] = intdec[1].substring(0,intdec[1].length() -1);  //shorten decimal part
  }
//  println(trim(intdec[0])+dsep+intdec[1]);
  if (float(intdec[1]) > 0)
    return intdec[0] + trim(dsep) + intdec[1];  //why is it necessary to trim dsep ? ?
    return intdec[0] + trim(dsep);  // + intdec[1];  //why is it necessary to trim dsep ? ?
    
}

String flToHr(float f, String symbol){
  String mul = "fpnµm KMGT";
  float i,v;
  int pmul;
  String s;
  String sign = " ";
  
//  println("flToHr:",f);
  if (f == 0) return "0" + symbol;
  if (f < 0) sign = "-";  // keep sign
  f = abs(f);  // ignore sign while searching for multiplier
  for (i = 1E12, pmul = 9;  pmul >= 0; i /= 1000, pmul -=1){
    if ((f/i <= 999) & (f/i >= 1)){
    f = (f/i);
//    s = str(f);
//    println(s);
    return sign + fixDsep(f,3,mul.substring(pmul,pmul+1) + symbol);  // + mul.substring(pmul,pmul+1) + symbol;
//      println(pmul);
//      println(mul.substring(pmul,pmul+1));
//      return sign + String.format("%.0f",f) + mul.charAt(pmul) + symbol;
    }
  }
  return "2BIG";
}  


void loadConfig(String file){  
    InputStream input;
    
  if (createInput(file) == null){  // If file not found, set some default values to start up
    config.set("winw","1024");
    config.set("winh","768");
    config.set("plotw","800");
    config.set("ploth","600");
    config.set("plotl","200");
    config.set("plott","10");
    config.set("fskX","0.5");
    config.set("fskY","0.5");
    config.set("grX","8");
    config.set("grY","8");
    config.set("grC","255,255,255");
    config.set("bckC","192,192,192");
    config.set("pwr","50");
    saveConfig(file);  // create the file
    return;            // that's all
  }
//  println("loadConfig");
// populate the config dictionary
  int i;
  String items[];
  String lines[] = loadStrings(file);  //Get all the lines of the file
  items = new String[2];  // Two items per line: key and value
  for (i = 0; i < lines.length; i++){  // for each line in lines
    items = split(lines[i]," ");
    if (items.length == 2) config.set(items[0],items[1]);  // else line is not interpretable
  }
/*  debug: show the content of config  
  String[] k = config.keyArray();
  for (i = 0; i < config.size(); i++){
    println(k[i],config.get(k[i]));
  } 
*/  
}

void saveConfig(String file){
//  println ("saveConfig");
  int i;
  String[] keyval;    // A key/value pair from the config dictionary
  String[] k = config.keyArray();  // The keys
//  println (k);
  keyval = new String[config.size()];
  for (i = 0; i < config.size(); i++) keyval[i] = k[i] + " " + config.get(k[i]);
  saveStrings(file,keyval);
}
  


public void draw(){
  float vx;
  float vy;
  
  if ((mouseX != pmouseX) | (mouseY != pmouseY)){  // Has moved, do something
    if (mouseX > 200 && mouseX < 200+800 && mouseY > 10 && mouseY < 10+600){  // inside spad
      //cursor(CROSS); //Someone restore the arrow ?
      vx = p.pixXToVal(mouseX-200);
      vy = p.pixYToVal(mouseY-10);
//      LbV.setText(str(vx));
      LbV.setText(flToHr(vx,"V"));
      LbV.setTextBold();
      
//      vy = round(vy*10000);  //vy is in Amps, round to 0.1 mA precision 
//      LbI.setText(str(vy/10));  // Display mA
      LbI.setText(flToHr(vy,"A"));
      LbI.setTextBold();
      LbP.setText(flToHr(vy*vx,"W"));
      LbP.setTextBold();
//      p.flyV(pmouseX,pmouseY);
//      p.flyV(mouseX, mouseY);
    } //else cursor(ARROW);
  }      
}  

//public boolean isMouseInto(){}

/*
  float vx;
  float vy;
  if (mousePressed && (mouseButton == LEFT)) {
    if (mouseX >= p.l && (mouseY >= p.t) && (mouseX < (p.l+p.w)) && (mouseY < (p.t+p.h))){
      // we are inside plot area, insert code here:
      vx = p.pixXToVal(mouseX);
      vy = p.pixYToVal(mouseY);
      LbV.setText(str(vx));
      flToHr(vx,"V");
      vy = round(vy*10000);  //vy is in Amps, round to 0.1 mA precision 
      LbI.setText(str(vy/10));  // Display mA 
    } 
  }
*/


